﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Web.Mvc;

namespace Harold.Net.Web.Mvc.Extension
{
    public class ConfiguredModelMetadata : ModelMetadata
    {
        private IEnumerable<ModelMetadata> _properties;
        
        private ModelMetadata _parentModelMetadata;

        private string _fullPropertyName = string.Empty;

        public ConfiguredModelMetadata(ModelMetadataProvider provider, Type containerType,
                                        Func<object> modelAccessor, Type modelType, string propertyName)
            : base(provider, containerType, modelAccessor, modelType, propertyName)
        {

        }

        public override IEnumerable<ModelValidator> GetValidators(ControllerContext context)
        {
            //return base.GetValidators(context);
            return ConfiguredModelValidatorProviders.Providers.GetValidators(this, context);
        }

        public override IEnumerable<ModelMetadata> Properties
        {
            get
            {
                _properties = _properties ?? ModelMetadataProviders.Current.GetMetadataForProperties(() => Model, this, this.ModelType, this.PropertyName);
                return _properties;
            }
        }

        public virtual ModelMetadata ParentModelMetadata
        {
            get
            {
                return _parentModelMetadata;
            }

            internal set
            {
                _parentModelMetadata = value;
            }
        }

        public virtual string FullPropertyName
        {
            get
            {
                return _fullPropertyName;

            }

            internal set
            {
                _fullPropertyName = value;
            }
        }
    }

    public static class ConfiguredModelValidatorProviders
    {
        private static readonly ModelValidatorProviderCollection _providers;

        static ConfiguredModelValidatorProviders()
        {
            _providers = new ModelValidatorProviderCollection() {
                new ConfiguredModelValidatorProvider(),
                new DataErrorInfoModelValidatorProvider(),
                new ClientDataTypeModelValidatorProvider()
            };
        }

        public static ModelValidatorProviderCollection Providers
        {
            get
            {
                return _providers;
            }
        }

    }

    public static class ModelMetadataExtension
    {
        public static void SetParentModelMetadata(this ModelMetadata instance, ModelMetadata parentModelMetadata)
        {
            var metadata = instance as ConfiguredModelMetadata;
            if (metadata != null)
            {
                metadata.ParentModelMetadata = parentModelMetadata;
            }
        }

        public static ModelMetadata GetParentModelMetadata(this ModelMetadata instance)
        {
            var metadata = instance as ConfiguredModelMetadata;
            if (metadata != null)
            {
                return metadata.ParentModelMetadata;
            }

            return null;
        }

        public static string GetFullPropertyName(this ModelMetadata instance)
        {
            var metadata = instance as ConfiguredModelMetadata;
            if (metadata != null)
            {
                return metadata.FullPropertyName;
            }

            return string.Empty;
        }

        public static void SetFullPropertyName(this ModelMetadata instance, string name)
        {
            var metadata = instance as ConfiguredModelMetadata;
            if (metadata != null)
            {
                metadata.FullPropertyName = name;
            }
        }
    }
}
