﻿using System;
using System.Collections.Generic;
using System.Web.Mvc;

using Microsoft.Practices.EnterpriseLibrary.Validation;

namespace Harold.Net.Web.Mvc.Extension
{
    public class EntlibModelValidator : ModelValidator
    {
        private Validator _innerValidator;

        public Validator InnerValidator
        {
            get
            {
                return _innerValidator;
            }
        }

        public EntlibModelValidator(ModelMetadata metadata, ControllerContext controllerContext, Validator innerValidator) :
            base(metadata, controllerContext) 
        {

            _innerValidator = innerValidator;
        }

        public override IEnumerable<ModelValidationResult> Validate(object container)
        {
            List<ModelValidationResult> results = new List<ModelValidationResult>();
            ValidationResults innerResults = new ValidationResults();
            //TypeDescriptor

            var objectToValidated = GetObjectToValidated();
            InnerValidator.DoValidate(objectToValidated, null, null, innerResults);
            foreach (ValidationResult result in innerResults)
            {
                results.Add(new ModelValidationResult { MemberName = Metadata.GetFullPropertyName(), Message = result.Message });
            }

            return results;
        }

        protected virtual object GetObjectToValidated()
        {
            if (Metadata.IsComplexType)
            {
                return Metadata.Model;
            }

            var rawValue = ControllerContext.Controller.ValueProvider.GetValue(Metadata.GetFullPropertyName());
            if (string.IsNullOrEmpty(rawValue.AttemptedValue))
            {
                return null;
            }

            var obj = ConvertProviderResult(rawValue, Metadata.ModelType);
            if (obj == null)
            {
                return rawValue.AttemptedValue;
            }

            return Metadata.Model;
        }

        private object ConvertProviderResult(ValueProviderResult valueProviderResult, Type destinationType)
        {
            try
            {
                object convertedValue = valueProviderResult.ConvertTo(destinationType);
                return convertedValue;
            }
            catch (Exception ex)
            {
                return null;
            }
        }
    }
}
