﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.Reflection;
using System.Web.Mvc;

namespace Harold.Net.Web.Mvc.Extension
{
    public static class ModelMetadataProviderExtension
    {
        public static ModelMetadata GetMetadataForType(this ModelMetadataProvider provider, Func<object> modelAccessor, ModelMetadata parentModelMetadata, Type modelType, string modelName)
        {
            var metadata = new ConfiguredModelMetadata(provider, parentModelMetadata != null ? parentModelMetadata.ModelType : null, modelAccessor, modelType, modelName);
            metadata.SetParentModelMetadata(parentModelMetadata);
            metadata.SetFullPropertyName(GetFullPropertyName(parentModelMetadata, metadata, modelName));

            return metadata;
        }

        public static IEnumerable<ModelMetadata> GetMetadataForProperties(this ModelMetadataProvider provider, Func<object> modelAccessor, ModelMetadata modelMetadata, Type modelType, string modelName)
        {
            var properties = modelType.GetProperties(BindingFlags.Public | BindingFlags.Instance);
            foreach(PropertyInfo property in properties)
            {
                object model = property.GetValue(modelMetadata.Model, null);
                yield return provider.GetMetadataForProperty(() => model, modelMetadata, property.PropertyType, property.Name);
            }
        }

        public static ModelMetadata GetMetadataForProperty(this ModelMetadataProvider provider, Func<object> modelAccessor, ModelMetadata parentModelMetadata, Type modelType, string modelName)
        {
            return provider.GetMetadataForType(modelAccessor, parentModelMetadata, modelType, modelName); ;
        }

        private static string GetFullPropertyName(ModelMetadata parentModelMetadata, ModelMetadata subModelMetadata, string modelName)
        {
            string fullName = string.Empty;

            if (parentModelMetadata != null)
            {
                fullName = parentModelMetadata.GetFullPropertyName();
                if (string.IsNullOrEmpty(fullName))
                {
                    fullName = modelName;
                }
                else
                {
                    fullName = string.Format("{0}.{1}", fullName, modelName);
                }
            }
            else
            {
                if (!subModelMetadata.IsComplexType)
                {
                   fullName = modelName;
                }
                else
                {
                    fullName = string.Empty;
                }
            }

            return fullName;
        }
    }
}
